@@ -29,14 +29,11 @@ import (
 )
 
 func TestClusterHealthCollector(t *testing.T) {
-	allVersions := []*Version{Nautilus, Octopus, Pacific}
-	nautilusOnly := []*Version{Nautilus}
-	octopusPlus := []*Version{Octopus, Pacific}
 	for _, tt := range []struct {
-		name     string
-		versions []*Version // Defaults to allVersions if not provided.
-		input    string
-		reMatch  []*regexp.Regexp
+		name    string
+		version string
+		input   string
+		reMatch []*regexp.Regexp
 	}{
 		{
 			name: "15 pgs stuck degraded",
@@ -44,6 +41,7 @@ func TestClusterHealthCollector(t *testing.T) {
 {
 	"health": {"summary": [{"severity": "HEALTH_WARN", "summary": "15 pgs stuck degraded"}]}
 }`,
+			version: `{"version":"ceph version 16.2.11-22-wasd (1984a8c33225d70559cdf27dbab81e3ce153f6ac) pacific (stable)"}`,
 			reMatch: []*regexp.Regexp{
 				regexp.MustCompile(`stuck_degraded_pgs{cluster="ceph"} 15`),
 			},
@@ -54,6 +52,7 @@ func TestClusterHealthCollector(t *testing.T) {
 {
 	"health": {"summary": [{"severity": "HEALTH_WARN", "summary": "16 pgs stuck unclean"}]}
 }`,
+			version: `{"version":"ceph version 16.2.11-22-wasd (1984a8c33225d70559cdf27dbab81e3ce153f6ac) pacific (stable)"}`,
 			reMatch: []*regexp.Regexp{
 				regexp.MustCompile(`stuck_unclean_pgs{cluster="ceph"} 16`),
 			},
@@ -64,6 +63,7 @@ func TestClusterHealthCollector(t *testing.T) {
 {
 	"health": {"summary": [{"severity": "HEALTH_WARN", "summary": "17 pgs stuck undersized"}]}
 }`,
+			version: `{"version":"ceph version 16.2.11-22-wasd (1984a8c33225d70559cdf27dbab81e3ce153f6ac) pacific (stable)"}`,
 			reMatch: []*regexp.Regexp{
 				regexp.MustCompile(`stuck_undersized_pgs{cluster="ceph"} 17`),
 			},
@@ -74,6 +74,7 @@ func TestClusterHealthCollector(t *testing.T) {
 {
 	"health": {"summary": [{"severity": "HEALTH_WARN", "summary": "18 pgs stuck stale"}]}
 }`,
+			version: `{"version":"ceph version 16.2.11-22-wasd (1984a8c33225d70559cdf27dbab81e3ce153f6ac) pacific (stable)"}`,
 			reMatch: []*regexp.Regexp{
 				regexp.MustCompile(`stuck_stale_pgs{cluster="ceph"} 18`),
 			},
@@ -84,6 +85,7 @@ func TestClusterHealthCollector(t *testing.T) {
 {
 	"pgmap": { "degraded_objects": 10 }
 }`,
+			version: `{"version":"ceph version 16.2.11-22-wasd (1984a8c33225d70559cdf27dbab81e3ce153f6ac) pacific (stable)"}`,
 			reMatch: []*regexp.Regexp{
 				regexp.MustCompile(`degraded_objects{cluster="ceph"} 10`),
 			},
@@ -94,13 +96,14 @@ func TestClusterHealthCollector(t *testing.T) {
 {
 	"pgmap": { "misplaced_objects": 20 }
 }`,
+			version: `{"version":"ceph version 16.2.11-22-wasd (1984a8c33225d70559cdf27dbab81e3ce153f6ac) pacific (stable)"}`,
 			reMatch: []*regexp.Regexp{
 				regexp.MustCompile(`misplaced_objects{cluster="ceph"} 20`),
 			},
 		},
 		{
-			name:     "10 down osds",
-			versions: nautilusOnly,
+			name:    "10 down osds",
+			version: `{"version":"ceph version 14.2.9-12-zasd (1337) pacific (stable)"}`,
 			input: `
 {
 	"osdmap": {
@@ -117,8 +120,8 @@ func TestClusterHealthCollector(t *testing.T) {
 			},
 		},
 		{
-			name:     "normal osdmap",
-			versions: nautilusOnly,
+			name:    "normal osdmap",
+			version: `{"version":"ceph version 14.2.9-12-zasd (1337) pacific (stable)"}`,
 			input: `
 {
 	"osdmap": {
@@ -139,8 +142,8 @@ func TestClusterHealthCollector(t *testing.T) {
 			},
 		},
 		{
-			name:     "10 down osds",
-			versions: octopusPlus,
+			name:    "10 down osds",
+			version: `{"version":"ceph version 16.2.11-22-wasd (1984a8c33225d70559cdf27dbab81e3ce153f6ac) pacific (stable)"}`,
 			input: `
 {
 	"osdmap": {
@@ -155,8 +158,8 @@ func TestClusterHealthCollector(t *testing.T) {
 			},
 		},
 		{
-			name:     "normal osdmap",
-			versions: octopusPlus,
+			name:    "normal osdmap",
+			version: `{"version":"ceph version 16.2.11-22-wasd (1984a8c33225d70559cdf27dbab81e3ce153f6ac) pacific (stable)"}`,
 			input: `
 {
 	"osdmap": {
@@ -175,49 +178,44 @@ func TestClusterHealthCollector(t *testing.T) {
 			},
 		},
 		{
-			name: "health ok",
-			input: `
-{
-	"health": { "status": "HEALTH_OK" } }`,
+			name:    "health ok",
+			input:   `{"health": { "status": "HEALTH_OK" } }`,
+			version: `{"version":"ceph version 16.2.11-22-wasd (1984a8c33225d70559cdf27dbab81e3ce153f6ac) pacific (stable)"}`,
 			reMatch: []*regexp.Regexp{
 				regexp.MustCompile(`health_status{cluster="ceph"} 0`),
 			},
 		},
 		{
-			name: "health warn",
-			input: `
-{
-	"health": { "status": "HEALTH_OK } }`,
+			name:    "health warn",
+			input:   `{"health": { "status": "HEALTH_OK" } }`,
+			version: `{"version":"ceph version 16.2.11-22-wasd (1984a8c33225d70559cdf27dbab81e3ce153f6ac) pacific (stable)"}`,
 			reMatch: []*regexp.Regexp{
 				regexp.MustCompile(`health_status{cluster="ceph"} 0`),
 				regexp.MustCompile(`health_status_interp{cluster="ceph"} 0`),
 			},
 		},
 		{
-			name: "health ok 2",
-			input: `
-{
-	"health": { "status": "HEALTH_OK } }`,
+			name:    "health ok 2",
+			input:   `{"health": { "status": "HEALTH_OK" } }`,
+			version: `{"version":"ceph version 16.2.11-22-wasd (1984a8c33225d70559cdf27dbab81e3ce153f6ac) pacific (stable)"}`,
 			reMatch: []*regexp.Regexp{
 				regexp.MustCompile(`health_status{cluster="ceph"} 0`),
 				regexp.MustCompile(`health_status_interp{cluster="ceph"} 0`),
 			},
 		},
 		{
-			name: "health warn 2",
-			input: `
-{
-	"health": { "status": "HEALTH_WARN" } }`,
+			name:    "health warn 2",
+			input:   `{"health": { "status": "HEALTH_WARN" } }`,
+			version: `{"version":"ceph version 16.2.11-22-wasd (1984a8c33225d70559cdf27dbab81e3ce153f6ac) pacific (stable)"}`,
 			reMatch: []*regexp.Regexp{
 				regexp.MustCompile(`health_status{cluster="ceph"} 1`),
 				regexp.MustCompile(`health_status_interp{cluster="ceph"} 2`),
 			},
 		},
 		{
-			name: "health err",
-			input: `
-{
-	"health": { "status": "HEALTH_ERR" } }`,
+			name:    "health err",
+			input:   `{"health": { "status": "HEALTH_ERR" } }`,
+			version: `{"version":"ceph version 16.2.11-22-wasd (1984a8c33225d70559cdf27dbab81e3ce153f6ac) pacific (stable)"}`,
 			reMatch: []*regexp.Regexp{
 				regexp.MustCompile(`health_status{cluster="ceph"} 2`),
 				regexp.MustCompile(`health_status_interp{cluster="ceph"} 3`),
@@ -233,6 +231,7 @@ $ sudo ceph -s
   recovery io 5779 MB/s, 4 keys/s, 1522 objects/s
   client io 4273 kB/s rd, 2740 MB/s wr, 2863 op/s
 `,
+			version: `{"version":"ceph version 16.2.11-22-wasd (1984a8c33225d70559cdf27dbab81e3ce153f6ac) pacific (stable)"}`,
 			reMatch: []*regexp.Regexp{
 				regexp.MustCompile(`recovery_io_bytes{cluster="ceph"} 5.779e`),
 				regexp.MustCompile(`recovery_io_keys{cluster="ceph"} 4`),
@@ -253,6 +252,7 @@ $ sudo ceph -s
   client io 2863 op/s rd, 5847 op/s wr
   cache io 251 MB/s flush, 6646 kB/s evict, 55 op/s promote
 `,
+			version: `{"version":"ceph version 16.2.11-22-wasd (1984a8c33225d70559cdf27dbab81e3ce153f6ac) pacific (stable)"}`,
 			reMatch: []*regexp.Regexp{
 				regexp.MustCompile(`recovery_io_bytes{cluster="ceph"} 5.779e`),
 				regexp.MustCompile(`recovery_io_keys{cluster="ceph"} 4`),
@@ -272,6 +272,7 @@ $ sudo ceph -s
 	"pgmap": { "num_pgs": 52000, "num_objects": 13156 },
 	"health": {"summary": [{"severity": "HEALTH_WARN", "summary": "7 pgs undersized"}]}
 }`,
+			version: `{"version":"ceph version 16.2.11-22-wasd (1984a8c33225d70559cdf27dbab81e3ce153f6ac) pacific (stable)"}`,
 			reMatch: []*regexp.Regexp{
 				regexp.MustCompile(`total_pgs{cluster="ceph"} 52000`),
 				regexp.MustCompile(`cluster_objects{cluster="ceph"} 13156`),
@@ -313,6 +314,7 @@ $ sudo ceph -s
 	},
 	"health": {"summary": [{"severity": "HEALTH_WARN", "summary": "7 pgs undersized"}]}
 }`,
+			version: `{"version":"ceph version 16.2.11-22-wasd (1984a8c33225d70559cdf27dbab81e3ce153f6ac) pacific (stable)"}`,
 			reMatch: []*regexp.Regexp{
 				regexp.MustCompile(`active_pgs{cluster="ceph"} 49`),
 				regexp.MustCompile(`scrubbing_pgs{cluster="ceph"} 2`),
@@ -339,6 +341,7 @@ $ sudo ceph -s
     }
   }
 }`,
+			version: `{"version":"ceph version 16.2.11-22-wasd (1984a8c33225d70559cdf27dbab81e3ce153f6ac) pacific (stable)"}`,
 			reMatch: []*regexp.Regexp{
 				regexp.MustCompile(`mons_down{cluster="ceph"} 1`),
 			},
@@ -356,6 +359,7 @@ $ sudo ceph -s
 		]
 	}
 }`,
+			version: `{"version":"ceph version 16.2.11-22-wasd (1984a8c33225d70559cdf27dbab81e3ce153f6ac) pacific (stable)"}`,
 			reMatch: []*regexp.Regexp{
 				regexp.MustCompile(`slow_requests{cluster="ceph"} 3`),
 			},
@@ -375,6 +379,7 @@ $ sudo ceph -s
     }
   }
 }`,
+			version: `{"version":"ceph version 16.2.11-22-wasd (1984a8c33225d70559cdf27dbab81e3ce153f6ac) pacific (stable)"}`,
 			reMatch: []*regexp.Regexp{
 				regexp.MustCompile(`slow_requests{cluster="ceph"} 3`),
 			},
@@ -394,6 +399,7 @@ $ sudo ceph -s
     }
   }
 }`,
+			version: `{"version":"ceph version 16.2.11-22-wasd (1984a8c33225d70559cdf27dbab81e3ce153f6ac) pacific (stable)"}`,
 			reMatch: []*regexp.Regexp{
 				regexp.MustCompile(`slow_requests{cluster="ceph"} 18`),
 			},
@@ -414,6 +420,7 @@ $ sudo ceph -s
   },
   "pgmap": { "degraded_objects": 154443937 }
 }`,
+			version: `{"version":"ceph version 16.2.11-22-wasd (1984a8c33225d70559cdf27dbab81e3ce153f6ac) pacific (stable)"}`,
 			reMatch: []*regexp.Regexp{
 				regexp.MustCompile(`degraded_objects{cluster="ceph"} 1.54443937e\+08`),
 				regexp.MustCompile(`health_status_interp{cluster="ceph"} 1`),
@@ -434,6 +441,7 @@ $ sudo ceph -s
     }
   }
 }`,
+			version: `{"version":"ceph version 16.2.11-22-wasd (1984a8c33225d70559cdf27dbab81e3ce153f6ac) pacific (stable)"}`,
 			reMatch: []*regexp.Regexp{
 				regexp.MustCompile(`new_crash_reports{cluster="ceph"} 2`),
 				regexp.MustCompile(`health_status_interp{cluster="ceph"} 1`),
@@ -454,6 +462,7 @@ $ sudo ceph -s
     }
   }
 }`,
+			version: `{"version":"ceph version 16.2.11-22-wasd (1984a8c33225d70559cdf27dbab81e3ce153f6ac) pacific (stable)"}`,
 			reMatch: []*regexp.Regexp{
 				regexp.MustCompile(`osds_too_many_repair{cluster="ceph"} 25`),
 				regexp.MustCompile(`health_status_interp{cluster="ceph"} 1`),
@@ -474,6 +483,7 @@ $ sudo ceph -s
     }
   }
 }`,
+			version: `{"version":"ceph version 16.2.11-22-wasd (1984a8c33225d70559cdf27dbab81e3ce153f6ac) pacific (stable)"}`,
 			reMatch: []*regexp.Regexp{
 				regexp.MustCompile(`health_status_interp{cluster="ceph"} 2`),
 			},
@@ -493,6 +503,7 @@ $ sudo ceph -s
     }
   }
 }`,
+			version: `{"version":"ceph version 16.2.11-22-wasd (1984a8c33225d70559cdf27dbab81e3ce153f6ac) pacific (stable)"}`,
 			reMatch: []*regexp.Regexp{
 				regexp.MustCompile(`osdmap_flag_full{cluster="ceph"} 0`),
 				regexp.MustCompile(`osdmap_flag_pauserd{cluster="ceph"} 1`),
@@ -510,6 +521,34 @@ $ sudo ceph -s
 				regexp.MustCompile(`health_status_interp{cluster="ceph"} 1`),
 			},
 		},
+		{
+			name: "many flags set new osdmap constmetrics filled",
+			input: `
+			{
+			  "health": {
+				"checks": {
+				  "OSDMAP_FLAGS": {
+					"severity": "HEALTH_WARN",
+					"summary": {
+						"message": "pauserd,pausewr,noout,noin,norecover,noscrub,notieragent,newhypotheticalcephflag flag(s) set; mon 482f68d873d2 is low on available space"
+					}
+				  }
+				}
+			  }
+			}`,
+			version: `{"version":"ceph version 16.2.11-22-wasd (1984a8c33225d70559cdf27dbab81e3ce153f6ac) pacific (stable)"}`,
+			reMatch: []*regexp.Regexp{
+				regexp.MustCompile(`osd_map_flags{cluster="ceph",flag="pauserd"} 1`),
+				regexp.MustCompile(`osd_map_flags{cluster="ceph",flag="pausewr"} 1`),
+				regexp.MustCompile(`osd_map_flags{cluster="ceph",flag="noin"} 1`),
+				regexp.MustCompile(`osd_map_flags{cluster="ceph",flag="noout"} 1`),
+				regexp.MustCompile(`osd_map_flags{cluster="ceph",flag="norecover"} 1`),
+				regexp.MustCompile(`osd_map_flags{cluster="ceph",flag="noscrub"} 1`),
+				regexp.MustCompile(`osd_map_flags{cluster="ceph",flag="notieragent"} 1`),
+				regexp.MustCompile(`osd_map_flags{cluster="ceph",flag="newhypotheticalcephflag"} 1`),
+				regexp.MustCompile(`health_status_interp{cluster="ceph"} 1`),
+			},
+		},
 		{
 			name: "lots of PG data",
 			input: `
@@ -611,6 +650,7 @@ $ sudo ceph -s
 		"bytes_total": 2537720565469184
 	}	
 }`,
+			version: `{"version":"ceph version 16.2.11-22-wasd (1984a8c33225d70559cdf27dbab81e3ce153f6ac) pacific (stable)"}`,
 			reMatch: []*regexp.Regexp{
 				regexp.MustCompile(`active_pgs{cluster="ceph"} 44`),
 				regexp.MustCompile(`degraded_pgs{cluster="ceph"} 40`),
@@ -657,8 +697,8 @@ $ sudo ceph -s
 			},
 		},
 		{
-			name:     "manager map",
-			versions: nautilusOnly,
+			name:    "manager map",
+			version: `{"version":"ceph version 14.2.9-12-zasd (1337) pacific (stable)"}`,
 			input: `
 {
     "mgrmap": {
@@ -708,8 +748,8 @@ $ sudo ceph -s
 			},
 		},
 		{
-			name:     "manager map",
-			versions: octopusPlus,
+			name:    "manager map",
+			version: `{"version":"ceph version 16.2.11-22-wasd (1984a8c33225d70559cdf27dbab81e3ce153f6ac) pacific (stable)"}`,
 			input: `
 {
     "mgrmap": {
@@ -767,6 +807,7 @@ $ sudo ceph -s
         }
     }
 }`,
+			version: `{"version":"ceph version 16.2.11-22-wasd (1984a8c33225d70559cdf27dbab81e3ce153f6ac) pacific (stable)"}`,
 			reMatch: []*regexp.Regexp{
 				regexp.MustCompile(`rbd_mirror_up{cluster="ceph",\s*name="prod-mon01-block01"} 1`),
 				regexp.MustCompile(`rbd_mirror_up{cluster="ceph",\s*name="prod-mon02-block01"} 1`),
@@ -774,38 +815,33 @@ $ sudo ceph -s
 		},
 	} {
 		t.Run(tt.name, func(t *testing.T) {
-			versions := allVersions
-			if len(tt.versions) > 0 {
-				versions = tt.versions
+			conn := setupVersionMocks(tt.version, "{}")
+			conn.On("MonCommand", mock.Anything).Return(
+				[]byte(tt.input), "", nil,
+			)
+			e := &Exporter{Conn: conn, Cluster: "ceph", Logger: logrus.New()}
+			e.cc = map[string]versionedCollector{
+				"clusterHealth": NewClusterHealthCollector(e),
 			}
-			for _, version := range versions {
-				t.Run(version.String(), func(t *testing.T) {
-					conn := &MockConn{}
-					conn.On("MonCommand", mock.Anything).Return(
-						[]byte(tt.input), "", nil,
-					)
 
-					collector := NewClusterHealthCollector(&Exporter{Conn: conn, Cluster: "ceph", Logger: logrus.New(), Version: version})
-					err := prometheus.Register(collector)
-					require.NoError(t, err)
-					defer prometheus.Unregister(collector)
+			err := prometheus.Register(e)
+			require.NoError(t, err)
+			defer prometheus.Unregister(e)
 
-					server := httptest.NewServer(promhttp.Handler())
-					defer server.Close()
+			server := httptest.NewServer(promhttp.Handler())
+			defer server.Close()
 
-					resp, err := http.Get(server.URL)
-					require.NoError(t, err)
-					defer resp.Body.Close()
+			resp, err := http.Get(server.URL)
+			require.NoError(t, err)
+			defer resp.Body.Close()
 
-					buf, err := ioutil.ReadAll(resp.Body)
-					require.NoError(t, err)
+			buf, err := ioutil.ReadAll(resp.Body)
+			require.NoError(t, err)
 
-					for _, re := range tt.reMatch {
-						if !re.Match(buf) {
-							t.Errorf("expected %s to match\n", re.String())
-						}
-					}
-				})
+			for _, re := range tt.reMatch {
+				if !re.Match(buf) {
+					t.Errorf("expected %s to match\n", re.String())
+				}
 			}
 		})
 	}
