@@ -19,7 +19,9 @@ import (
 	"encoding/json"
 	"fmt"
 	"math"
+	"strconv"
 	"strings"
+	"sync"
 	"time"
 
 	"github.com/prometheus/client_golang/prometheus"
@@ -28,21 +30,20 @@ import (
 
 const (
 	osdLabelFormat = "osd.%v"
-)
 
-const (
 	scrubStateIdle          = 0
 	scrubStateScrubbing     = 1
 	scrubStateDeepScrubbing = 2
+
+	oldestInactivePGUpdatePeriod = 10 * time.Second
 )
 
 // OSDCollector displays statistics about OSD in the Ceph cluster.
 // An important aspect of monitoring OSDs is to ensure that when the cluster is
 // up and running that all OSDs that are in the cluster are up and running, too
 type OSDCollector struct {
-	conn    Conn
-	logger  *logrus.Logger
-	version *Version
+	conn   Conn
+	logger *logrus.Logger
 
 	// osdScrubCache holds the cache of previous PG scrubs
 	osdScrubCache map[int]int
@@ -54,9 +55,6 @@ type OSDCollector struct {
 	// a PG to not have an active state in it.
 	oldestInactivePGMap map[string]time.Time
 
-	// pgDumpBrief holds the content of PG dump brief
-	pgDumpBrief cephPGDumpBrief
-
 	// CrushWeight is a persistent setting, and it affects how CRUSH assigns data to OSDs.
 	// It displays the CRUSH weight for the OSD
 	CrushWeight *prometheus.GaugeVec
@@ -102,6 +100,9 @@ type OSDCollector struct {
 	// OSDUp displays the Up state of the OSD
 	OSDUp *prometheus.GaugeVec
 
+	// OSDMetaData displays metadata of an OSD
+	OSDMetadata *prometheus.GaugeVec
+
 	// OSDFullRatio displays current full_ratio of OSD
 	OSDFullRatio prometheus.Gauge
 
@@ -152,20 +153,17 @@ type OSDCollector struct {
 	OldestInactivePG prometheus.Gauge
 }
 
-// This ensures OSDCollector implements interface prometheus.Collector.
-var _ prometheus.Collector = &OSDCollector{}
-
 // NewOSDCollector creates an instance of the OSDCollector and instantiates the
 // individual metrics that show information about the OSD.
 func NewOSDCollector(exporter *Exporter) *OSDCollector {
 	labels := make(prometheus.Labels)
 	labels["cluster"] = exporter.Cluster
 	osdLabels := []string{"osd", "device_class", "host", "rack", "root"}
+	osdMetadataLabels := []string{"osd", "objectstore", "ceph_version_when_created", "created_at"}
 
-	return &OSDCollector{
-		conn:    exporter.Conn,
-		logger:  exporter.Logger,
-		version: exporter.Version,
+	o := &OSDCollector{
+		conn:   exporter.Conn,
+		logger: exporter.Logger,
 
 		osdScrubCache:       make(map[int]int),
 		osdLabelsCache:      make(map[int64]*cephOSDLabel),
@@ -402,6 +400,16 @@ func NewOSDCollector(exporter *Exporter) *OSDCollector {
 			osdLabels,
 		),
 
+		OSDMetadata: prometheus.NewGaugeVec(
+			prometheus.GaugeOpts{
+				Namespace:   cephNamespace,
+				Name:        "osd_metadata",
+				Help:        "OSD Metadata",
+				ConstLabels: labels,
+			},
+			osdMetadataLabels,
+		),
+
 		OSDDownDesc: prometheus.NewDesc(
 			fmt.Sprintf("%s_osd_down", cephNamespace),
 			"Number of OSDs down in the cluster",
@@ -442,6 +450,9 @@ func NewOSDCollector(exporter *Exporter) *OSDCollector {
 			},
 		),
 	}
+
+	go o.oldestInactivePGLoop()
+	return o
 }
 
 func (o *OSDCollector) collectorList() []prometheus.Collector {
@@ -464,6 +475,7 @@ func (o *OSDCollector) collectorList() []prometheus.Collector {
 		o.ApplyLatency,
 		o.OSDIn,
 		o.OSDUp,
+		o.OSDMetadata,
 		o.OSDFullRatio,
 		o.OSDNearFullRatio,
 		o.OSDBackfillFullRatio,
@@ -586,6 +598,13 @@ type cephOSDLabel struct {
 	parent      int64   // parent id when building tables
 }
 
+type cephOSDMetadata struct {
+	ID                     int    `json:"id"`
+	CephVersionWhenCreated string `json:"ceph_version_when_created"`
+	CreatedAt              string `json:"created_at"`
+	OsdObjectstore         string `json:"osd_objectstore"`
+}
+
 func (o *OSDCollector) collectOSDDF() error {
 	args := o.cephOSDDFCommand()
 	buf, _, err := o.conn.MgrCommand(args)
@@ -705,6 +724,29 @@ func (o *OSDCollector) collectOSDDF() error {
 
 }
 
+func (o *OSDCollector) collectOSDMetadata() error {
+	cmd := o.cephOSDMetadataCommand()
+	buf, _, err := o.conn.MonCommand(cmd)
+	if err != nil {
+		o.logger.WithError(err).WithField(
+			"args", string(cmd),
+		).Error("error executing mon command")
+
+		return err
+	}
+
+	var osdMetadata []cephOSDMetadata
+	if err := json.Unmarshal(buf, &osdMetadata); err != nil {
+		return err
+	}
+
+	for _, osd := range osdMetadata {
+		o.OSDMetadata.WithLabelValues(strconv.Itoa(osd.ID), osd.OsdObjectstore, osd.CephVersionWhenCreated, osd.CreatedAt).Set(1)
+	}
+
+	return nil
+}
+
 func (o *OSDCollector) collectOSDPerf() error {
 	args := o.cephOSDPerfCommand()
 	buf, _, err := o.conn.MgrCommand(args)
@@ -874,8 +916,8 @@ func (o *OSDCollector) collectOSDTreeDown(ch chan<- prometheus.Metric) error {
 			osdName,
 			lb.DeviceClass,
 			lb.Host,
-			lb.Root,
-			lb.Rack)
+			lb.Rack,
+			lb.Root)
 	}
 
 	return nil
@@ -955,7 +997,7 @@ func (o *OSDCollector) collectOSDDump() error {
 
 }
 
-func (o *OSDCollector) performPGDumpBrief() error {
+func (o *OSDCollector) performPGDumpBrief() (*cephPGDumpBrief, error) {
 	args := o.cephPGDumpCommand()
 	buf, _, err := o.conn.MgrCommand(args)
 	if err != nil {
@@ -963,18 +1005,23 @@ func (o *OSDCollector) performPGDumpBrief() error {
 			"args", string(bytes.Join(args, []byte(","))),
 		).Error("error executing mgr command")
 
-		return err
+		return nil, err
 	}
 
-	o.pgDumpBrief = cephPGDumpBrief{}
-	if err := json.Unmarshal(buf, &o.pgDumpBrief); err != nil {
-		return err
+	pgDumpBrief := cephPGDumpBrief{}
+	if err := json.Unmarshal(buf, &pgDumpBrief); err != nil {
+		return nil, err
 	}
 
-	return nil
+	return &pgDumpBrief, nil
 }
 
 func (o *OSDCollector) collectOSDScrubState(ch chan<- prometheus.Metric) error {
+	pgDumpBrief, err := o.performPGDumpBrief()
+	if err != nil {
+		return err
+	}
+
 	// need to reset the PG scrub state since the scrub might have ended within
 	// the last prom scrape interval.
 	// This forces us to report scrub state on all previously discovered OSDs We
@@ -984,7 +1031,7 @@ func (o *OSDCollector) collectOSDScrubState(ch chan<- prometheus.Metric) error {
 		o.osdScrubCache[i] = scrubStateIdle
 	}
 
-	for _, pg := range o.pgDumpBrief.PGStats {
+	for _, pg := range pgDumpBrief.PGStats {
 		if strings.Contains(pg.State, "scrubbing") {
 			scrubState := scrubStateScrubbing
 			if strings.Contains(pg.State, "deep") {
@@ -1006,7 +1053,7 @@ func (o *OSDCollector) collectOSDScrubState(ch chan<- prometheus.Metric) error {
 			fmt.Sprintf(osdLabelFormat, i),
 			lb.DeviceClass,
 			lb.Host,
-			lb.Root,
+			lb.Rack,
 			lb.Root)
 	}
 
@@ -1046,6 +1093,17 @@ func (o *OSDCollector) cephOSDPerfCommand() [][]byte {
 	return [][]byte{cmd}
 }
 
+func (o *OSDCollector) cephOSDMetadataCommand() []byte {
+	cmd, err := json.Marshal(map[string]interface{}{
+		"prefix": "osd metadata",
+		"format": jsonFormat,
+	})
+	if err != nil {
+		o.logger.WithError(err).Panic("error marshalling ceph osd metadata")
+	}
+	return cmd
+}
+
 func (o *OSDCollector) cephOSDTreeCommand(states ...string) []byte {
 	req := map[string]interface{}{
 		"prefix": "osd tree",
@@ -1074,36 +1132,46 @@ func (o *OSDCollector) cephPGDumpCommand() [][]byte {
 	return [][]byte{cmd}
 }
 
-func (o *OSDCollector) collectPGStates(ch chan<- prometheus.Metric) error {
-	// - See if there are PGs that we're tracking that are now active
-	// - See if there are new ones to add
-	// - Find the oldest one
-	now := time.Now()
-	oldestTime := now
-
-	for _, pg := range o.pgDumpBrief.PGStats {
-		// If we were tracking it, and it's now active, remove it
-		active := strings.Contains(pg.State, "active")
-		if active {
-			delete(o.oldestInactivePGMap, pg.PGID)
+func (o *OSDCollector) oldestInactivePGLoop() {
+	for {
+		pgDumpBrief, err := o.performPGDumpBrief()
+		if err != nil {
+			o.logger.WithError(err).Warning("failed to get latest PG dump for oldest inactive PG update")
+			time.Sleep(oldestInactivePGUpdatePeriod)
 			continue
 		}
 
-		// Now see if it's not here, we'll need to track it now
-		pgTime, ok := o.oldestInactivePGMap[pg.PGID]
-		if !ok {
-			pgTime = now
-			o.oldestInactivePGMap[pg.PGID] = now
+		// - See if there are PGs that we're tracking that are now active
+		// - See if there are new ones to add
+		// - Find the oldest one
+		now := time.Now()
+		oldestTime := now
+
+		for _, pg := range pgDumpBrief.PGStats {
+			// If we were tracking it, and it's now active, remove it
+			active := strings.Contains(pg.State, "active")
+			if active {
+				delete(o.oldestInactivePGMap, pg.PGID)
+				continue
+			}
+
+			// Now see if it's not here, we'll need to track it now
+			pgTime, ok := o.oldestInactivePGMap[pg.PGID]
+			if !ok {
+				pgTime = now
+				o.oldestInactivePGMap[pg.PGID] = now
+			}
+
+			// And finally, track our oldest time
+			if pgTime.Before(oldestTime) {
+				oldestTime = pgTime
+			}
 		}
 
-		// And finally, track our oldest time
-		if pgTime.Before(oldestTime) {
-			oldestTime = pgTime
-		}
+		o.OldestInactivePG.Set(float64(now.Unix() - oldestTime.Unix()))
+
+		time.Sleep(oldestInactivePGUpdatePeriod)
 	}
-
-	o.OldestInactivePG.Set(float64(now.Unix() - oldestTime.Unix()))
-	return nil
 }
 
 // Describe sends the descriptors of each OSDCollector related metrics we have
@@ -1119,8 +1187,8 @@ func (o *OSDCollector) Describe(ch chan<- *prometheus.Desc) {
 
 // Collect sends all the collected metrics to the provided Prometheus channel.
 // It requires the caller to handle synchronization.
-func (o *OSDCollector) Collect(ch chan<- prometheus.Metric) {
-	// Reset daemon specifc metrics; daemons can leave the cluster
+func (o *OSDCollector) Collect(ch chan<- prometheus.Metric, version *Version) {
+	// Reset daemon specific metrics; daemons can leave the cluster
 	o.CrushWeight.Reset()
 	o.Depth.Reset()
 	o.Reweight.Reset()
@@ -1134,42 +1202,60 @@ func (o *OSDCollector) Collect(ch chan<- prometheus.Metric) {
 	o.ApplyLatency.Reset()
 	o.OSDIn.Reset()
 	o.OSDUp.Reset()
+	o.OSDMetadata.Reset()
 	o.buildOSDLabelCache()
 
-	o.logger.Debug("collecting OSD perf metrics")
-	if err := o.collectOSDPerf(); err != nil {
-		o.logger.WithError(err).Error("error collecting OSD perf metrics")
-	}
+	localWg := &sync.WaitGroup{}
 
-	o.logger.Debug("collecting OSD dump metrics")
-	if err := o.collectOSDDump(); err != nil {
-		o.logger.WithError(err).Error("error collecting OSD dump metrics")
-	}
+	localWg.Add(1)
+	go func() {
+		defer localWg.Done()
+		if err := o.collectOSDPerf(); err != nil {
+			o.logger.WithError(err).Error("error collecting OSD perf metrics")
+		}
+	}()
 
-	o.logger.Debug("collecting OSD df metrics")
-	if err := o.collectOSDDF(); err != nil {
-		o.logger.WithError(err).Error("error collecting OSD df metrics")
-	}
+	localWg.Add(1)
+	go func() {
+		defer localWg.Done()
+		if err := o.collectOSDMetadata(); err != nil {
+			o.logger.WithError(err).Error("error collecting OSD metadata metrics")
+		}
+	}()
 
-	o.logger.Debug("collecting OSD tree down metrics")
-	if err := o.collectOSDTreeDown(ch); err != nil {
-		o.logger.WithError(err).Error("error collecting OSD tree down metrics")
-	}
+	localWg.Add(1)
+	go func() {
+		defer localWg.Done()
+		if err := o.collectOSDDump(); err != nil {
+			o.logger.WithError(err).Error("error collecting OSD dump metrics")
+		}
+	}()
 
-	o.logger.Debug("collecting PG dump metrics")
-	if err := o.performPGDumpBrief(); err != nil {
-		o.logger.WithError(err).Error("error collecting PG dump metrics")
-	}
+	localWg.Add(1)
+	go func() {
+		defer localWg.Done()
+		if err := o.collectOSDDF(); err != nil {
+			o.logger.WithError(err).Error("error collecting OSD df metrics")
+		}
+	}()
 
-	o.logger.Debug("collecting OSD scrub metrics")
-	if err := o.collectOSDScrubState(ch); err != nil {
-		o.logger.WithError(err).Error("error collecting OSD scrub metrics")
-	}
+	localWg.Add(1)
+	go func() {
+		defer localWg.Done()
+		if err := o.collectOSDTreeDown(ch); err != nil {
+			o.logger.WithError(err).Error("error collecting OSD tree down metrics")
+		}
+	}()
 
-	o.logger.Debug("collecting PG states")
-	if err := o.collectPGStates(ch); err != nil {
-		o.logger.WithError(err).Error("error collecting PG state metrics")
-	}
+	localWg.Add(1)
+	go func() {
+		defer localWg.Done()
+		if err := o.collectOSDScrubState(ch); err != nil {
+			o.logger.WithError(err).Error("error collecting OSD scrub metrics")
+		}
+	}()
+
+	localWg.Wait()
 
 	for _, metric := range o.collectorList() {
 		metric.Collect(ch)
